from django.db import models
import qrcode
from io import BytesIO
from django.core.files.base import ContentFile
from django.contrib.auth import get_user_model

User = get_user_model()
class Bank(models.Model):
    name = models.CharField(max_length=255, unique=True)

    def __str__(self):
        return self.name


class Event(models.Model):
    ORGANISATION_TYPES = [
        ('Individual', 'Individual'),
        ('Company', 'Company'),
    ]
    GSTIN_STATUSES = [
        ('Yes', 'Yes'),
        ('No', 'No'),
    ]
    EVENT_TYPES = [
        ('offline', 'Offline'),
        ('online', 'Online'),
    ]
    
    organisation_type = models.CharField(max_length=50, choices=ORGANISATION_TYPES)
    organisation_name = models.CharField(max_length=255, blank=True, null=True)
    full_name = models.CharField(max_length=255,blank=True, null=True)
    mobile_number = models.CharField(max_length=15, blank=True, null=True)
    event_type = models.CharField(max_length=20, choices=EVENT_TYPES, default='offline')
    pan_number = models.CharField(max_length=15)
    address = models.TextField()
    gstin_status = models.CharField(max_length=10, choices=GSTIN_STATUSES)
    gst_number = models.CharField(max_length=20, blank=True, null=True)
    email = models.EmailField()
    bank = models.ForeignKey(Bank, on_delete=models.CASCADE, related_name="events")
    beneficiary_name = models.CharField(max_length=255)
    account_number = models.CharField(max_length=20)
    ifsc_code = models.CharField(max_length=15)
    title = models.CharField(max_length=255)
    overview = models.TextField()
    location = models.CharField(max_length=255)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    total_tickets = models.PositiveIntegerField()
    available_tickets = models.PositiveIntegerField(default=0)
    event_category = models.CharField(max_length=100, blank=True, null=True)
    event_date = models.DateField()
    event_time = models.TimeField()
    upload_pan = models.FileField(upload_to='uploads/pan/')
    upload_cancelled_cheque = models.FileField(upload_to='uploads/cancelled_cheques/')
    thumbnail = models.ImageField(upload_to='uploads/thumbnails/', blank=True, null=True)
    created_by = models.ForeignKey(User, on_delete=models.CASCADE, related_name='events')
    status = models.IntegerField(default=1)

    def __str__(self):
        return self.title


#     def __str__(self):
#         return f"Booking {self.booking_id} for {self.event.title}"

class Booking(models.Model):
    PAYMENT_STATUS_CHOICES = [
        ('success', 'Success'),
        ('failed', 'Failed'),
    ]

    event = models.ForeignKey(Event, on_delete=models.CASCADE, related_name="bookings")
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="bookings")
    ticket_count = models.PositiveIntegerField()
    amount_paid = models.DecimalField(max_digits=10, decimal_places=2)
    transaction_id = models.CharField(max_length=255, blank=True, null=True)
    payment_status = models.CharField(max_length=10, choices=PAYMENT_STATUS_CHOICES)
    qr_code = models.ImageField(upload_to='uploads/qrcodes/', blank=True, null=True)
    booking_id = models.CharField(max_length=20, unique=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        # Generate QR code for booking
        qr_data = f"Booking ID: {self.booking_id}\nEvent: {self.event.title}\nTickets: {self.ticket_count}\nAmount Paid: {self.amount_paid}\nTransaction ID: {self.transaction_id}"
        qr = qrcode.QRCode(version=1, box_size=10, border=4)
        qr.add_data(qr_data)
        qr.make(fit=True)
        img = qr.make_image(fill='black', back_color='white')

        # Save QR code to file
        buffer = BytesIO()
        img.save(buffer)
        buffer.seek(0)
        self.qr_code.save(f"{self.booking_id}_qrcode.png", ContentFile(buffer.read()), save=False)
        buffer.close()

        super().save(*args, **kwargs)